#!/usr/bin/env python3
# © Copyright 2020 - 2021 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates and licensors
# (“Micro Focus”) are as may be set forth in the express warranty statements accompanying such
# products and services. Nothing herein should be construed as constituting an additional
# warranty. Micro Focus shall not be liable for technical or editorial errors or omissions
# contained herein. The information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential information
# and a valid license is required for possession, use or copying. If this work is provided to the
# U.S. Government, consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items are licensed
# to the U.S. Government under vendor's standard commercial license.

"""
Python script to fetch SMS data for OES telemetry.
"""

import glob
import json
import os
import subprocess
import sys
import time
from collections import OrderedDict

"""
service - Name of the service
properties - Systemctl properties
result - Final result format of SMS telemetry data.
telemetry_result_loc - Location to place SMS data for telemetry
telemetry_config - Telemetry config file's location
"""

service = "novell-smdrd"
properties = ["ActiveState"]
result = OrderedDict([
    ("OESService", "OES Backup / Storage Managament Services (SMS)"),
    ("timestamp", int(time.time()))])
telemetry_result_loc = "/var/opt/novell/telemetry/data/"
telemetry_config = "/etc/opt/novell/telemetry/config/telemetry.json"


def get_systemctl_properties():
    """
    Function to run systemctl and capture the output.
    :return: dict: Result of systemctl
    """

    systemctl_result = dict()
    cmd = "systemctl show " + service + " --property=" + ",".join(properties)
    output = run_command(cmd)

    for line in output.splitlines():
        key, value = line.strip().split("=")
        systemctl_result[key] = value

    return systemctl_result


def run_command(cmd):
    """
    Function to execute linux command and capture the result
    :param cmd: linux command
    :return: Output of the command
    """

    err, output = None, None
    ret = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    try:
        (output, err) = ret.communicate(timeout=5)
    except TimeoutError:
        ret.kill()

    if err:
        sys.exit(2)
    elif output:
        return output.decode('utf-8')


def add_tsa_list():
    """
    Function to add loaded TSAs to result.
    """

    cmd = "/opt/novell/sms/bin/smsconfig -t | tail -n +2"
    tsa_list = run_command(cmd)
    if tsa_list:
        result["TSAs"] = tsa_list.split()


def add_tlsversion():
    """
    Function to add TLS version to result
    """

    cmd = "/usr/bin/grep '^[[:blank:]]*[^[:blank:]#;]' /etc/opt/novell/sms/smdrd.conf | grep -w 'tlsversion' |  awk '{print $2}'"
    tlsversion = run_command(cmd)
    if not tlsversion:
        tlsversion = "tlsv1_0"
    result["TLS version"] = tlsversion.strip()


def add_error(msg):
    """
    Function to add error to result
    :param msg: Error message
    """
    if "Error" in result:
        result["Error"] += [msg]
    else:
        result["Error"] = [msg]


def write_result():
    """
    Function to remove old, create new and update the file with result.
    """

    files = glob.glob(telemetry_result_loc + 'SMS_*.json')
    for file in files:
        try:
            os.remove(file)
        except OSError as e:
            add_error("Unable to delete old data file: " + e.strerror)

    file_name = "SMS_" + str(result["timestamp"]) + ".json"
    file_path = telemetry_result_loc + file_name
    file = open(file_path, "w")
    file.write(json.dumps(result))
    file.close()


def telemetry_check():
    """
    To check if telemetry is enabled.
    :return: Boolean
    """

    try:
        with open(telemetry_config, "r") as file:
            status = json.load(file)
            if status["opt-in"].lower() == "yes":
                return True
    except IOError:
        pass

    return False


def generate_sms_data():
    """
    The main function to generate the data of SMS for Telemetry.

    :return 0 on success
    """

    # generate only if telemetry is enabled
    enabled = telemetry_check()
    if enabled:
        service_properties = get_systemctl_properties()
        if service_properties != -1:
            state = service_properties.get("ActiveState")
            if state == "active":
                add_tsa_list()
                add_tlsversion()
        write_result()

    return 0


# Script starts from here
if __name__ == '__main__':
    generate_sms_data()
